#include <stddef.h>
#include <arduino.h>
#include "ArenaLighting.h"

/*
 * Initialize the Arena lighting.
 */
ArenaLighting::ArenaLighting(byte dataPin, byte storagePin, byte clockPin) {
  _dataPin = dataPin;
  _storagePin = storagePin;
  _clockPin = clockPin;

  // Initialize digital pins for the SN74HC565.
  pinMode(_dataPin, OUTPUT);
  pinMode(_storagePin, OUTPUT);
  pinMode(_clockPin, OUTPUT);
};


/*
 * Switches all LEDs off.
 */
void ArenaLighting::off(){
  on(LED_NONE);
}


/*
 * Switch selected LEDs on or off.

   Examples for 16 bit integer values for <leds>.
    0xFFFF - 11111111 11111111
    0x1419 - 00010100 00011001
 */
void ArenaLighting::on(int leds){
    // Split the integer into two bytes.
    byte right = (leds >> 8) & 0xFF;
    byte left = leds & 0xFF;

    // Pass these bytes to the SN74HC595
    digitalWrite(_storagePin, LOW);
    shiftOut(_dataPin, _clockPin, MSBFIRST, left);  
    shiftOut(_dataPin, _clockPin, MSBFIRST, right);  
    digitalWrite(_storagePin, HIGH);
}


/*
 * Switch selected LEDs on or off for milliseconds.
 */
void ArenaLighting::on(int leds, int milliseconds) {
  on(leds);
  delay(milliseconds);
  off();
}


/*
 * Lighting program for Arena opening.
 * 
 * Umnlaufend wird jede einzelne LED kurz eingeschaltet.
 */
void ArenaLighting::boot(){
  // First of all, switch all LEDs off.
  off();

  // 8 rounds: every LED is switched on for 50 ms.
  for(byte r=0;r<8;r++) {
    for(byte i=0;i<16;i++) {
      on(1 << i, 50);
    }
  }

  // All LEDs are turned on for 1,5 seconds.
  on(LED_ALL, 2500);

  // Den LED Spot noch 2 Sekunden leuchten lassen.
  on(LED_SPOT, 1000);

  // Switch all LEDs off.
  off();
}

 
/*
 * Start Battle 
 */
void ArenaLighting::startBattle() {
  // First of all, switch all LEDs off.
  off();

  // beide roten LEDs für 1 Sekunde einschalten, danach kurz warten.
  on(LED_RED, 1000);
  delay(200);

  // beide roten LEDs für 1 Sekunde einschalten, danach kurz warten.
  on(LED_RED, 1000);
  delay(200);

  // beide gelben LEDs für 1 Sekunde einschalten, danach kurz warten.
  on(LED_YELLOW, 1000);
  delay(200);

  // beide gelben LEDs für 1 Sekunde einschalten, danach kurz warten.
  on(LED_YELLOW, 1000);
  delay(200);

  // beide grünen LEDs für 2 Sekunden und den Spot einschalten.
  on(LED_GREEN + LED_SPOT, 2000);

  // LED Spot eingeschaltet lassen für 2 weitere Sekunden.
  on(LED_SPOT, 2000);

  // Switch all LEDs off.
  off();
}


/*
 * Ring out finish
 */
void ArenaLighting::ringOutFinish(){
 // First of all, switch all LEDs off.
  off();

  // 10 rounds: every LED is switched on for 25 ms.
  for(byte r=0; r<10; r++) {
    for(byte i=0; i<16; i++) {
      on(1 << i);
      delay(25);
    }
  }
  
  // Switch all LEDs off.
  off();
}


/*
 * Survivor finish
 */
void ArenaLighting::survivorFinish(){
  // First of all, switch all LEDs off.
  off();

  for(byte i=0; i<10; i++) {
    on(LED_LEFT, 200);
    on(LED_RIGHT, 200);
  }
  
  // Switch all LEDs off.
  off();
}


/*
 * Burst finisch
 */
void ArenaLighting::burstFinish(){
  // First of all, switch all LEDs off.
  off();

  // beide weißen LEDs blinken lassen
  for(byte i=0; i<4; i++) {
    on(LED_WHITE, 200);
    on(LED_NONE, 100);
  }

  // Alle LEDs einschalten für 2 Sekunden
  int leds = LED_ALL;
  on(leds, 2000);

  // beide weißen LEDs ausschalten
  leds -= LED_WHITE;
  on(leds, 750);

  // beide roten LEDs ausschalten
  leds -= LED_RED;
  on(leds, 750);
  
  // beide gelben LEDs ausschalten
  leds -= LED_YELLOW;
  on(leds, 750);
  
  // beide grüne LEDs ausschalten
  leds -= LED_GREEN;
  on(leds, 750);
  
  // beide blauen LEDs ausschalten
  leds -= LED_BLUE;
  on(leds, 1000);

  // Switch all LEDs off.
  off();
}

/*
void ArenaLighting::fadeIn(int leds, int milliseconds){
  for(int i = 0; i<500; i++) {
    on(0x0001);
    delay(2);
    off();
    delay(10);
  }
}
*/
